package org.processmining.logenhancement.abstraction.model;

import java.util.HashSet;
import java.util.Set;

import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public final class AbstractionPattern {

	private final String patternName;
	private final DataPetriNet net;

	private Transition startTransition;
	private Transition completeTransition;
	//TODO add other lifecycle transitions

	private Set<Transition> transitions = new HashSet<Transition>();

	private Place cardinalityPlace;

	public AbstractionPattern(String patternName, DataPetriNet net) {
		super();
		this.patternName = patternName;
		this.net = net;
	}

	public String getPatternName() {
		return patternName;
	}

	public Set<Transition> getTransitions() {
		return transitions;
	}

	public Set<String> getTransitionsAsLocalIds() {
		Set<String> localIds = new HashSet<String>();
		for (Transition t : getTransitions()) {
			localIds.add(t.getLocalID().localNodeID().toString());
		}
		return localIds;
	}

	public Transition getStartTransition() {
		return startTransition;
	}

	public void setStartTransition(Transition startTransition) {
		this.startTransition = startTransition;
	}

	public Transition getCompleteTransition() {
		return completeTransition;
	}

	public void setCompleteTransition(Transition completeTransition) {
		this.completeTransition = completeTransition;
	}

	public Place getCardinalityPlace() {
		return cardinalityPlace;
	}

	public void setCardinalityPlace(Place cardinalityPlace) {
		this.cardinalityPlace = cardinalityPlace;
	}

	public DataPetriNet getDPN() {
		return net;
	}

}