package org.processmining.logenhancement.enriching;

import java.util.Date;
import java.util.ListIterator;

import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;

public class AddDurationTimePlugin {
	@Plugin(name = "Add Time between events (Duration) as Attribute to all Events", parameterLabels = { "A log" }, //
			help = "Adds a new attribute (continuous) with the time between events to all events.", categories = {
					PluginCategory.Enhancement }, returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " Alfredo Bolt", email = "a.bolt@tue.nl")
	public void addTimeBetweenEvents(UIPluginContext context, XLog log) throws UserCancelledException {

		Progress progBar = context.getProgress();
		progBar.setMinimum(0);
		progBar.setMaximum(log.size());
		progBar.setValue(0);

		String elapsedTimeKey = ProMUIHelper.queryForString(context, "Specify name of the attribute to be created");
		TimeResolution resolution = ProMUIHelper.queryForObject(context,
				"Specify the resolution to be used for the time between events", TimeResolution.values());

		doAddTimeBetweenEvents(progBar, log, elapsedTimeKey, resolution);
	}

	public void doAddTimeBetweenEvents(Progress progress, XLog log, String timeBetweenEventsKey,
			TimeResolution resolution) {
		ListIterator<XTrace> traceIterator = log.listIterator();
		XFactory factory = XFactoryRegistry.instance().currentDefault();

		while (traceIterator.hasNext()) {
			XTrace trace = traceIterator.next();

			long previousTime = -1;

			ListIterator<XEvent> eventIter = trace.listIterator();
			while (eventIter.hasNext()) {
				XEvent event = eventIter.next();
				Date eventTime = XTimeExtension.instance().extractTimestamp(event);
				if (eventTime != null) {
					if (previousTime < 0) { //first event has time 0
						event.getAttributes().put(timeBetweenEventsKey, factory.createAttributeContinuous(
								timeBetweenEventsKey, (0 / (double) resolution.getFactor()), null));
						previousTime = eventTime.getTime();
					} else { //not the first event
						event.getAttributes().put(timeBetweenEventsKey,
								factory.createAttributeContinuous(timeBetweenEventsKey,
										((eventTime.getTime() - previousTime) / (double) resolution.getFactor()), null));
						previousTime = eventTime.getTime();
					}
				} else
					throw new RuntimeException("An event in trace is missing timestamp!");
			}
			progress.inc();
		}
	}

}