package org.processmining.logenhancement.enriching;

import java.util.Date;
import java.util.ListIterator;

import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;

public class AddThroughputTimePlugin {

	@Plugin(name = "Add Throughput Time as Trace Attribute (In place)", parameterLabels = { "A log" }, //
			returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void mergeEvents(UIPluginContext context, XLog log) throws UserCancelledException {

		Progress progBar = context.getProgress();

		TimeResolution resolution = ProMUIHelper.queryForObject(context,
				"Specify the resolution to be used for the elapsed time", TimeResolution.values());

		doAddThroughputTime(log, resolution, progBar);
	}

	private void doAddThroughputTime(XLog log, TimeResolution resolution, Progress progress) {

		progress.setMinimum(0);
		progress.setMaximum(log.size());
		progress.setValue(0);

		ListIterator<XTrace> traceIterator = log.listIterator();
		XFactory factory = XFactoryRegistry.instance().currentDefault();

		while (traceIterator.hasNext()) {
			XTrace trace = traceIterator.next();

			long startTime = Long.MAX_VALUE;
			long endTime = 0;

			ListIterator<XEvent> eventIterator = trace.listIterator();
			while (eventIterator.hasNext()) {
				Date eventTime = XTimeExtension.instance().extractTimestamp(eventIterator.next());
				startTime = Math.min(startTime, eventTime.getTime());
				endTime = Math.max(startTime, eventTime.getTime()); // minimum is start time
			}

			double elapsedTime = endTime - startTime;
			trace.getAttributes().put("throughputtime",
					factory.createAttributeContinuous("throughputtime", (elapsedTime / resolution.getFactor()), null));

			progress.inc();
		}
	}

}
