package org.processmining.logenhancement.enriching;

import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;

public class CascadeAttributesPlugin {

	@Plugin(name = "Cascade Attributes to Events in a Trace", parameterLabels = { "A log" }, //
			help = "Copies attributes to subsequent events if they do not have a value for such attribute", categories = {
					PluginCategory.Enhancement }, returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " Alfredo Bolt", email = "a.bolt@tue.nl")
	public void cascadeEventsInTrace(UIPluginContext context, XLog log) {

		Progress progBar = context.getProgress();
		progBar.setMinimum(0);
		progBar.setMaximum(log.size());
		progBar.setValue(0);

		doCascadeEventsInTrace(progBar, log);
	}

	public void doCascadeEventsInTrace(Progress progress, XLog log) {
		for (XTrace trace : log) {
			XEvent previousEvent = null;
			for (XEvent event : trace) {
				if (previousEvent == null) {
					previousEvent = event;
					continue;
				}
				XAttributeMap previousAttributes = previousEvent.getAttributes();
				XAttributeMap attributes = event.getAttributes();
				for (String attName : previousAttributes.keySet()) {
					if (attributes.containsKey(attName) && attributes.get(attName) != null
							&& !attributes.get(attName).toString().isEmpty()) {
						//do nothing, because there is a valid attribute there!
					} else 
						//(over)write the attribute
						attributes.put(attName, previousAttributes.get(attName));
				}
				previousEvent = event;
			}
			progress.inc();
		}
	}

}