package org.processmining.logenhancement.filtering;

import java.util.Random;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XLogImpl;
import org.deckfour.xes.model.impl.XTraceImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.ui.widgets.ProMTextField;

public class RemoveEventsFromLogProbabilityBased {


	@Plugin(name = "Remove events from log traces",
			parameterLabels = { "Log"},
			returnLabels = { "XLog"},
			returnTypes = { XLog.class },
			userAccessible = true,
			help = "Adds noise to an event log by removing random events.")
			@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "M. de Leoni", email = "m.d.leoni@tue.nl")
	public XLog plugin(UIPluginContext context, XLog log) 
	{
		XLog newLog=new XLogImpl((XAttributeMap) log.getAttributes().clone());
		ProMTextField field=new ProMTextField();
		InteractionResult result = context.showConfiguration("Specify the probability (between 1 and 99) of an event to be removed", field);
		int probability=0;
		do
		{
			if (result == InteractionResult.CANCEL) {
				context.getFutureResult(0).cancel(true);
				return null;
			}
			try
			{
				probability=Integer.parseInt(field.getText());
			}
			catch(NumberFormatException err)
			{ }
		} while(probability <=0 || probability>=100);
		Random randomGenerator=new Random();
		context.getProgress().setMaximum(log.size());
		for(XTrace trace : log)
		{
			XTrace newTrace=new XTraceImpl((XAttributeMap) trace.getAttributes().clone());
			for(XEvent event : trace)
			{
				if(randomGenerator.nextInt(100)+1>probability)
					newTrace.add(event);
			}
			newLog.add(newTrace);
			context.getProgress().inc();
		}
		return newLog;
	}
}
