package org.processmining.logenhancement.merge.alignmentbased;

import gnu.trove.map.TIntObjectMap;
import gnu.trove.map.hash.TIntObjectHashMap;
import gnu.trove.procedure.TIntObjectProcedure;

import java.util.BitSet;
import java.util.Iterator;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XTrace;
import org.processmining.logenhancement.merge.alignmentbased.timeconstraints.TimeConstraints;
import org.processmining.plugins.DataConformance.Alignment;

interface TraceSet {
	
	final static class TraceEntry {

		private final XTrace trace;
		private final Alignment alignment;
		private Candidates candidates;

		public TraceEntry(XTrace trace, Alignment alignment, Candidates candidates) {
			this.trace = trace;
			this.alignment = alignment;
			this.candidates = candidates;
		}

		public XTrace getTrace() {
			return trace;
		}

		public String getTraceName() {
			return XConceptExtension.instance().extractName(trace);
		}

		public Alignment getAlignment() {
			return alignment;
		}

		public Candidates getCandidates() {
			return candidates;
		}

		public void setCandidates(Candidates candidates) {
			this.candidates = candidates;
		}

		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((getTraceName() == null) ? 0 : getTraceName().hashCode());
			return result;
		}

		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			TraceEntry other = (TraceEntry) obj;
			if (getTraceName() == null) {
				if (other.getTraceName() != null)
					return false;
			} else if (!getTraceName().equals(other.getTraceName()))
				return false;
			return true;
		}

		public String toString() {
			return getTraceName();
		}

	}
	
	final static class Candidates {

		private boolean shouldRetry = false;
		private TIntObjectMap<TimeConstraints> timeConstraints;
		private TIntObjectMap<BitSet> eventMatches;

		Candidates() {
			super();
			this.timeConstraints = new TIntObjectHashMap<TimeConstraints>();
			this.eventMatches = new TIntObjectHashMap<BitSet>();
		}

		public void addMatch(int index, int eventIndex) {
			createOrGet(index).set(eventIndex);
		}

		private BitSet createOrGet(int index) {
			BitSet orderedList = eventMatches.get(index);
			if (orderedList == null) {
				orderedList = new BitSet();
				eventMatches.put(index, orderedList);
			}
			return orderedList;
		}

		public TIntObjectMap<EventEntry> createResult(final EventSet eventSet) {
			final TIntObjectMap<EventEntry> resultMatches = new TIntObjectHashMap<EventEntry>(eventMatches.size());
			eventMatches.forEachEntry(new TIntObjectProcedure<BitSet>() {

				public boolean execute(int index, BitSet result) {
					assert result.cardinality() <= 1 : "Trying to create a ResultSet with ambigous matches!";
					if (!result.isEmpty()) {
						resultMatches.put(index, eventSet.getEvent(result.nextSetBit(0)));
					}
					return true;
				}
			});
			return resultMatches;
		}

		public void addTimeConstraint(int index, TimeConstraints constraint) {
			timeConstraints.put(index, constraint);
		}

		public boolean isShouldRetry() {
			return shouldRetry;
		}

		public void setShouldRetry(boolean shouldRetry) {
			this.shouldRetry = shouldRetry;
		}

		public TIntObjectMap<BitSet> getEventMatches() {
			return eventMatches;
		}

		public void setEventMatches(TIntObjectMap<BitSet> eventMatches) {
			this.eventMatches = eventMatches;
		}

		public TIntObjectMap<TimeConstraints> getTimeConstraints() {
			return timeConstraints;
		}

		public void setTimeConstraints(TIntObjectMap<TimeConstraints> timeConstraints) {
			this.timeConstraints = timeConstraints;
		}

	}

	XTrace getByName(String name);

	boolean markRetry(XTrace t, Alignment alignment, Candidates candidates);

	boolean hasRetries();

	int retryCount();

	Iterator<TraceEntry> retryIterator();

}