package org.processmining.logenhancement.transforming;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;
import org.processmining.log.utils.XUtils;
import org.processmining.logenhancement.utils.LogEnhancementHelper;

import com.google.common.collect.Sets;

public class ChangeAttributeName {

	@Plugin(name = "Change attribute name (In place)", parameterLabels = { "A log" }, //
			help = "Changes the name of an attribute.", categories = {
					PluginCategory.Enhancement }, returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void changeAttributeType(UIPluginContext context, XLog log) throws UserCancelledException {

		context.getProgress().setMaximum(log.size());

		XEventClassifier classifier = LogEnhancementHelper.queryEventClassifier(context, log);

		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, classifier);
		Set<XEventClass> eventNames = Sets.newTreeSet(logInfo.getEventClasses().getClasses());

		List<XEventClass> selectedEventClasses = ProMUIHelper.queryForObjects(context,
				"Please select the event classes for which the attribute name should be changed", eventNames);

		List<String> attributeKeys = new ArrayList<>(logInfo.getEventAttributeInfo().getAttributeKeys());
		Collections.sort(attributeKeys);

		String attributeName = ProMUIHelper.queryForString(context,
				"Please select the attribute that should be changed", attributeKeys);

		String changedName = ProMUIHelper.queryForString(context, "Please provide the new name of the attribute",
				attributeName);

		XFactory factory = XFactoryRegistry.instance().currentDefault();

		doChangeAttributeName(factory, log, logInfo, selectedEventClasses, attributeName, changedName);

	}

	public void doChangeAttributeName(XFactory factory, XLog log, XLogInfo logInfo,
			List<XEventClass> selectedEventClasses, String attributeName, String newAttributeName) {
		for (XTrace t : log) {
			for (XEvent e : t) {
				if (selectedEventClasses.contains(logInfo.getEventClasses().getClassOf(e))) {
					XAttribute attribute = e.getAttributes().remove(attributeName);
					if (attribute != null) {
						XUtils.putAttribute(e,
								XUtils.cloneAttributeWithChangedKeyWithFactory(attribute, newAttributeName, factory));
					}
				}
			}
		}
	}

}
