package org.processmining.logenhancement.transforming;

import java.text.MessageFormat;
import java.util.Set;
import java.util.TreeSet;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;
import org.processmining.logenhancement.utils.LogEnhancementHelper;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;

public class ChangeEventNames {

	@Plugin(name = "Change concept:name based on another attribute", parameterLabels = { "Event Log" }, //
			categories = {
					PluginCategory.Enhancement }, help = "Changes the 'concept:name' attribute of events based on the value of another attribute for the same event. For example, it is possible to concatenate two attributes and use the result as new 'concept:name'.", returnLabels = {
							"Log with changed 'concept:name'" }, returnTypes = { XLog.class }, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public XLog changeEventNameBasedOnAttribute(UIPluginContext context, XLog log) throws UserCancelledException {

		XEventClassifier classifier = LogEnhancementHelper.queryEventClassifier(context, log);
		if (!(classifier.getDefiningAttributeKeys().length == 1
				&& classifier.getDefiningAttributeKeys()[0].equals("concept:name"))) {
			throw new RuntimeException("This plug-in does only work with the XEventNameClassifier (concept:name)");
		}
		XEventClasses eventClasses = XEventClasses.deriveEventClasses(classifier, log);
		Set<XEventClass> eventNames = Sets.newTreeSet(eventClasses.getClasses());

		Set<XEventClass> selectedEventClass = ImmutableSet
				.copyOf(ProMUIHelper.queryForObjects(context, "Please choose event classes", eventNames));

		Set<String> attributes = new TreeSet<>();
		for (XTrace t : log) {
			for (XEvent e : t) {
				if (selectedEventClass.contains(eventClasses.getClassOf(e))) {
					for (XAttribute a : e.getAttributes().values()) {
						attributes.add(a.getKey());
					}
				}
			}
		}

		String attributeName = ProMUIHelper.queryForObject(context, "Please choose an attribute", attributes);

		String newNameFormat = ProMUIHelper.queryForString(context, "Please specify new concept:name format",
				"Use {0} as placeholder for the current event name, and {1} as placeholder for the attribute value.",
				"{0} - {1}");

		XFactory factory = XFactoryRegistry.instance().currentDefault();

		return doChangeEventName(log, eventClasses, factory, selectedEventClass, attributeName, newNameFormat);
	}

	public XLog doChangeEventName(XLog log, XEventClasses eventClasses, XFactory factory, Set<XEventClass> selectedEventClass,
			String attributeName, String newNameFormat) {
		XLog newLog = factory.createLog((XAttributeMap) log.getAttributes().clone());

		for (XTrace t : log) {
			XTrace newTrace = factory.createTrace((XAttributeMap) t.getAttributes().clone());

			for (XEvent e : t) {
				XEvent newEvent = (XEvent) e.clone();
				newTrace.add(newEvent);

				if (selectedEventClass.contains(eventClasses.getClassOf(e))) {

					String oldName = XConceptExtension.instance().extractName(e);

					XAttribute selectedAttribute = e.getAttributes().get(attributeName);
					if (selectedAttribute != null) {
						XConceptExtension.instance().assignName(newEvent,
								MessageFormat.format(newNameFormat, oldName, selectedAttribute.toString()));
					}

				}
			}

			newLog.add(newTrace);
		}

		return newLog;
	}

}
