package org.processmining.logenhancement.transforming;

import java.util.Set;
import java.util.TreeSet;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;
import org.processmining.logenhancement.utils.LogEnhancementHelper;

import com.google.common.collect.Sets;

public class ChangeEventTimestamp {

	@Plugin(name = "Change time:timestamp based on attribute", parameterLabels = {
			"Event Log" }, categories = {
					PluginCategory.Enhancement }, help = "Changes the 'time:timestamp' of an event based on another attribute.", returnLabels = {
							"Log with changed 'time:timestamp'" }, returnTypes = { XLog.class }, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public XLog changeEventTimestampBasedOnAttribute(UIPluginContext context, XLog log) throws UserCancelledException {

		XEventClassifier classifier = LogEnhancementHelper.queryEventClassifier(context, log);

		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, classifier);
		Set<XEventClass> eventNames = Sets.newTreeSet(logInfo.getEventClasses().getClasses());
		XEventClass selectedEventClass = ProMUIHelper.queryForObject(context, "Please choose event class", eventNames);

		Set<String> attributes = new TreeSet<>();
		for (XTrace t : log) {
			for (XEvent e : t) {
				if (selectedEventClass.equals(logInfo.getEventClasses().getClassOf(e))) {
					for (XAttribute a : e.getAttributes().values()) {
						if (a instanceof XAttributeTimestamp) {
							attributes.add(a.getKey());
						}
					}
				}
			}
		}

		String attributeName = ProMUIHelper.queryForObject(context, "Please choose an attribute", attributes);

		XFactory factory = XFactoryRegistry.instance().currentDefault();

		return doChangeEventTimestamp(log, logInfo, factory, selectedEventClass, attributeName);
	}

	public XLog doChangeEventTimestamp(XLog log, XLogInfo logInfo, XFactory factory, XEventClass selectedEventClass,
			String attributeName) {
		XLog newLog = factory.createLog((XAttributeMap) log.getAttributes().clone());

		for (XTrace t : log) {
			XTrace newTrace = factory.createTrace((XAttributeMap) t.getAttributes().clone());

			for (XEvent e : t) {
				XEvent newEvent = (XEvent) e.clone();

				if (selectedEventClass.equals(logInfo.getEventClasses().getClassOf(e))) {
					XAttribute selectedAttribute = e.getAttributes().get(attributeName);
					if (selectedAttribute instanceof XAttributeTimestamp) {
						XTimeExtension.instance().assignTimestamp(newEvent,
								((XAttributeTimestamp) selectedAttribute).getValue());
					}
				}

				newTrace.insertOrdered(newEvent);
			}

			newLog.add(newTrace);
		}

		return newLog;
	}

}
