package org.processmining.logenhancement.transforming;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Map.Entry;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;
import org.processmining.log.utils.XUtils;
import org.processmining.logenhancement.utils.LogEnhancementHelper;

import com.google.common.collect.ImmutableMap;

public class CopyAttributesToTracePlugin {

	@Plugin(name = "Copy attribute to trace level (In place)", parameterLabels = { "Event Log" }, //
	returnLabels = {}, returnTypes = {}, categories = { PluginCategory.Enhancement }, userAccessible = true, mostSignificantResult = -1, //
	help = "Copies attributes to the trace level, i.e., creates an attribute with the last value of that attribute in events at the trace level.")
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void copySelectedAttributeToTrace(UIPluginContext context, XLog log) throws UserCancelledException {

		Progress progBar = context.getProgress();
		progBar.setMinimum(0);
		progBar.setMaximum(log.size());
		progBar.setValue(0);

		XEventClassifier classifier = LogEnhancementHelper.queryEventClassifier(context, log);
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, classifier);

		List<String> attributeKeys = new ArrayList<>(logInfo.getEventAttributeInfo().getAttributeKeys());
		Collections.sort(attributeKeys);
		Map<String, String> attributeNames = ImmutableMap.copyOf(ProMUIHelper.queryMapToString(context,
				"Select attribute that should be copied", attributeKeys));

		ListIterator<XTrace> traceIterator = log.listIterator();

		while (traceIterator.hasNext()) {

			XTrace trace = traceIterator.next();
			ListIterator<XEvent> eventIterator = trace.listIterator();

			while (eventIterator.hasNext()) {
				int eventIndex = eventIterator.nextIndex();
				XEvent event = eventIterator.next();

				for (Entry<String, String> attributeEntry : attributeNames.entrySet()) {
					String oldKey = attributeEntry.getKey();
					XAttribute attribute = event.getAttributes().get(oldKey);
					if (attribute != null) {
						String newKey = attributeEntry.getValue();
						trace.getAttributes().put(newKey, XUtils.cloneAttributeWithChangedKey(attribute, newKey));
					}
				}

				trace.set(eventIndex, event);
			}
			progBar.inc();
		}

	}

}
