package org.processmining.logenhancement.transforming;

import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;
import org.processmining.log.utils.XUtils;
import org.processmining.logenhancement.utils.LogEnhancementHelper;

public class MoveEventAttributePlugin {

	@Plugin(name = "Move an attribute between events (In place)", parameterLabels = { "A log" }, //
			categories = {
					PluginCategory.Enhancement }, help = "Moves an attribute from a source event class to a target event class.", returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void moveAttributes(UIPluginContext context, XLog log) throws UserCancelledException {

		XEventClassifier classifier = LogEnhancementHelper.queryEventClassifier(context, log);

		XEventClasses eventClasses = XUtils.createEventClasses(classifier, log);
		Set<String> eventAttributeKeys = XUtils.getEventAttributeKeys(log);

		XEventClass sourceEvent = ProMUIHelper.queryForObject(context,
				"Select the source event class (1st event of this class will be used)", eventClasses.getClasses());
		XEventClass targetEvent = ProMUIHelper.queryForObject(context, "Select the target event class",
				eventClasses.getClasses());

		String attributeName = ProMUIHelper.queryForObject(context, "Select the attribute that should be moved",
				eventAttributeKeys);
		String newAttributeName = ProMUIHelper.queryForString(context, "Specify the 'name' of the new attribute", null,
				attributeName);

		doMoveAttribute(log, eventClasses, sourceEvent, targetEvent, attributeName, newAttributeName);

	}

	public void doMoveAttribute(XLog log, XEventClasses eventClasses, XEventClass sourceEvent, XEventClass targetEvent,
			String attributeName, String newAttributeName) {
		for (XTrace xTrace : log) {

			XAttribute sourceAttribute = null;
			for (XEvent e : xTrace) {
				if (isSameEventClass(eventClasses, sourceEvent, e)) {
					sourceAttribute = e.getAttributes().remove(attributeName);
					// Just take the first occurrence
					break;
				}
			}

			for (XEvent e : xTrace) {
				if (isSameEventClass(eventClasses, targetEvent, e)) {
					if (sourceAttribute != null) {
						e.getAttributes().put(newAttributeName,
								XUtils.cloneAttributeWithChangedKey(sourceAttribute, newAttributeName));
					}
				}
			}

		}
	}

	private boolean isSameEventClass(XEventClasses eventClasses, XEventClass sourceEvent, XEvent e) {
		return sourceEvent.equals(eventClasses.getClassOf(e));
	}

}
