package org.processmining.logenhancement.transforming;

import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;

public class RepairLogMetadataPlugin {

	public enum RepairType {
		LIFECYCLE_COMPLETE_IF_MISSING, LIFECYCLE_START_IF_MISSING, LIFECYCLE_COMPLETE_OVERWRITE, LIFECYCLE_START_OVERWRITE
	}

	@Plugin(name = "Add life-cycle information to events (In place)", parameterLabels = { "Event Log" }, returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1,
			categories = { PluginCategory.Enhancement },
			help = "Repairs the Event Log by adding/removing attributes such as the lifecycle:transition, ...")
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void repairLog(final UIPluginContext context, XLog eventLog) throws UserCancelledException {

		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(eventLog.size());

		final RepairType type = ProMUIHelper.queryForObject(context, "Choose how 'lifecycle:transition' of every event should be changed", RepairType.values());

		XLifecycleExtension lifecycle = XLifecycleExtension.instance();
		eventLog.getExtensions().add(lifecycle);
		
		for (XTrace t : eventLog) {

			for (XEvent e : t) {
				switch (type) {
					case LIFECYCLE_COMPLETE_IF_MISSING :
						if (lifecycle.extractStandardTransition(e) == null) {
							lifecycle.assignStandardTransition(e, StandardModel.COMPLETE);	
						}						
						break;
						
					case LIFECYCLE_START_IF_MISSING :
						if (lifecycle.extractStandardTransition(e) == null) {
							lifecycle.assignStandardTransition(e, StandardModel.START);	
						}						
						break;
						
					case LIFECYCLE_COMPLETE_OVERWRITE :
						lifecycle.assignStandardTransition(e, StandardModel.COMPLETE);	
						break;
						
					case LIFECYCLE_START_OVERWRITE :
						lifecycle.assignStandardTransition(e, StandardModel.START);	
						break;

					default :
						break;
				}
			}

		}

	}

}
