package org.processmining.logenhancement.transforming;

import java.util.Random;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XTraceImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.log.utils.XUtils;

public class SwapEvents {

	@Plugin(name = "Swap events in log traces", parameterLabels = { "Log" }, returnLabels = { "XLog" }, returnTypes = {
			XLog.class }, userAccessible = true, help = "Add noise to an event log by swapping events with a certain probability.")
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "M. de Leoni", email = "m.d.leoni@tue.nl")
	public XLog plugin(UIPluginContext context, XLog log) {
		XLog newLog = XUtils.createLogFrom(log);

		ProMTextField field = new ProMTextField();
		InteractionResult result = context
				.showConfiguration("Specify the probability (between 1 and 99) of an event to be swapped", field);
		int probability = 0;
		do {
			if (result == InteractionResult.CANCEL) {
				context.getFutureResult(0).cancel(true);
				return null;
			}
			try {
				probability = Integer.parseInt(field.getText());
			} catch (NumberFormatException err) {
			}
		} while (probability <= 0 || probability >= 100);
		Random randomGenerator = new Random();
		context.getProgress().setMaximum(log.size());
		for (XTrace trace : log) {
			XEvent[] eventArray = trace.toArray(new XEvent[0]);
			if (trace.size() > 1) { //FM, this does not work on traces with just one event
				boolean[] swappedPosition = new boolean[eventArray.length];
				for (int i = 0; i < swappedPosition.length; i++)
					swappedPosition[i] = false;
				for (int i = 0; i < eventArray.length; i++) {
					if (randomGenerator.nextInt(100) + 1 <= probability) {
						int pos;
						do {
							pos = randomGenerator.nextInt(eventArray.length);
						} while (pos == i);
						if (swappedPosition[pos] == true) {
							do {
								pos = randomGenerator.nextInt(eventArray.length);
							} while (pos == i);
						}
						if (swappedPosition[pos] == false) {
							swappedPosition[pos] = true;
							swappedPosition[i] = true;
							XEvent app = eventArray[i];
							eventArray[i] = eventArray[pos];
							eventArray[pos] = app;
						}
					}
				}
			}
			XTrace newTrace = new XTraceImpl((XAttributeMap) trace.getAttributes().clone());
			for (XEvent newEvent : eventArray)
				newTrace.add(newEvent);
			newLog.add(newTrace);
			context.getProgress().inc();
		}
		return newLog;
	}

}
