package org.processmining.logenhancement.view;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.List;

import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.log.utils.XUtils;

import com.google.common.collect.Multiset.Entry;

final class ActionGroupExport implements ActionListener {

	private final LogViewMaster masterView;
	private final LogViewContext context;

	ActionGroupExport(LogViewMaster masterView, LogViewContext context) {
		this.masterView = masterView;
		this.context = context;
	}

	public void actionPerformed(ActionEvent e) {
		List<Entry<XESTraceVariant>> selection = masterView.getMasterList().getList().getSelectedValuesList();

		XFactory factory = XFactoryRegistry.instance().currentDefault();
		XLog newLog = factory.createLog();
		XUtils.assignConceptName(newLog, proposeName(selection));

		int groupIndex = 0;
		for (Entry<XESTraceVariant> group : selection) {
			Collection<XTrace> detailTraces = masterView.getDetailElements(group);
			for (XTrace trace : detailTraces) {
				XTrace clonedTrace = (XTrace) trace.clone();
				addMetaData(clonedTrace, groupIndex, group.getCount());
				newLog.add(clonedTrace);
			}
			groupIndex++;
		}

		context.exportLog(newLog);
	}

	private void addMetaData(XTrace detailTraces, int groupIndex, int groupCount) {
		XUtils.putAttribute(detailTraces, XUtils.createAttribute("variant:index", Long.valueOf(groupIndex)));
		XUtils.putAttribute(detailTraces, XUtils.createAttribute("variant:count", Long.valueOf(groupCount)));
	}

	private String proposeName(List<Entry<XESTraceVariant>> selection) {
		if (selection.size() != 1) {
			return "Multiple Trace Variants";
		} else {
			return selection.get(0).getCount() + " Traces Variant";
		}
	}
}